#!/usr/bin/env python

import sys,time
from myutils import *
from Split import *
from string import *

def parse_options():
    from optparse import OptionParser, OptionGroup
    parser = OptionParser()
    group=OptionGroup(parser,"Input options")
    group.add_option("-s", "--source", dest="source", help="Filename for source trees in newick format",default="")
    group.add_option("-t", "--taxa", dest="taxa", help="Taxafilename: each taxon one line, default no file, read out of gene trees (ignored if smatrix given)",default="")
    group.add_option("-z", "--smatrix", dest="smatrix", help="Source trees as matrix representation in nexus format (ignored if source trees given)",default="")
    group.add_option("-r","--tree", dest="stree",help="File with supertrees in newick format",default="")
    parser.add_option_group(group)

    group=OptionGroup(parser,"Output options")
    group.add_option("-o", "--head", dest="header", help="Header for output (default mrc)",default="")
    group.add_option("-v", "--time", dest="time", help="Verbose mode with printing times to stdout (default)",action="store_false",default=True)
    parser.add_option_group(group)

    (options, args) = parser.parse_args()
    return options,args

# compute and return the score
def score_super(super,genes): 
    comp_super(super,genes) #replace scores with new supertree
    return get_score(genes)

# compute the scores and thet them in the Split objects of the gene trees
def comp_super(super,genes):
    l=len(super)
    for gene in genes: #gene is SplitSet
        for g in gene:
            g.score=[False for i in range(0,l)]
            for s in super:
                i=s.index
                g.score[i]=g.subsplit(s.split)

# compute a distance from the scores
def get_score(gset):
    return sum([sum([1-i.get_score() for i in g]) for g in gset])    

# represent time
def difftime(start,format):
    return time.strftime(format,time.gmtime(time.time()-start))

# find taxa in gene tree strings
def find_taxa(glist):
    taxa=set([])
    for tree in glist:
        taxa=taxa.union(set(parse_newick(tree).get_leaves()))
    taxa=list(taxa)
    taxa.sort()
    return taxa

# read the matrix representation out of a nexus file, also the weights from the paupblock are included
def get_matrix_from_nexus(smatrix):
    import re
    gset=""
    inf=open(smatrix).readlines()
    l=len(inf)
    i=0
    start=re.compile("matrix", re.IGNORECASE)
    while start.search(inf[i])==None:
        i+=1
    i+=1
    alg=[]
    j=inf[i].find(";")
    while j==-1: 
        alg.append(inf[i].split())
        i+=1
        j=inf[i].find(";")
    last=inf[i][:j].split()
    if len(last)==2:alg.append(last)
    SplitSet.taxa=[a[0] for a in alg]
    gset=[]
    for j in range(0,len(alg[0][1])):
        gset.append(Split([a[1][j] for a in alg]))

    start=re.compile("weights", re.IGNORECASE)
    j=start.match(inf[i]) #match only beginning of line!
    while j==None:
        i+=1
        if i==l:return [gset] #no weights
        j=start.match(inf[i])

    p=re.compile('(.*?)\:(.*?)(,|;)')
    found=p.findall(inf[i][8:]) #start after weights
    for f in found:
        w,r=float(f[0]),f[1]
        j=r.find("-")
        if j==-1: #no range, gset index starts with 0!
            gset[int(r)-1].set_weight(w)
        else:
            s,e=int(r[:j]),int(r[j+1:])
            for k in range(s,e+1):
                gset[k-1].set_weight(w)
    s=sum([g.weight for g in gset])
    print len(gset),s
    fact=len(gset)/s #factor, such that weights sum up to matrix length -> on avg. weight 1
    for g in gset:
        g.weight=g.weight*fact
    return [gset]
    
def mrc(opts):

    if opts.time:
        starttime=time.time()
        timeformat="%Hh %Mm %Ss"
    else:
        starttime=0
        timeformat=""

    if starttime: print difftime(starttime,timeformat), "Start reading source trees"

    ########################### Read source trees or matrix ###########################
    term=True
    empty=True

    if opts.source:
        gene=get_treelist(opts.source)
        print "No of gene trees",len(gene)
        if opts.taxa:
            SplitSet.taxa=[i[:-1] for i in open(opts.taxa).readlines()]
        else:
            SplitSet.taxa=find_taxa(gene)
        gset=[SplitSet(g,term=term,empty=empty) for g in gene]
        
    elif opts.smatrix:
        # the whole matrix is read as one tree, does not change mrc score
        gset=get_matrix_from_nexus(opts.smatrix)

    else:
        print "Error no gene trees given (option -s or -z)"
        sys.exit(0)


    Split.max=2**(len(SplitSet.taxa))-1 #all positions 1
    print "Taxa:",len(SplitSet.taxa),SplitSet.taxa

    if starttime: print difftime(starttime,timeformat), "Finished reading source trees"

    algmap={"m":"mrc"}
    if not opts.header: header="mrc"
    else:header=opts.header
    

    ############### Evaluate Trees and Finish #########################################

    if starttime: print difftime(starttime,timeformat),"Start evaluation of user trees"
    outp=open(header+".eval",'w') #for scores
    trees=get_treelist(opts.stree)
    
    for tree in trees:
        sset=SplitSet(tree,term,empty) #also read term and empty since they are not filtered from the matrix representation
        score=score_super(sset,gset)
        if type(score)==type(0):outp.write("%i\n" %score)
        else:outp.write("%1.4f\n" %score)
    
    outp.close()
    if starttime: print difftime(starttime,timeformat),"Finished evaluation of user trees"

    scores=[float(s) for s in open(header+".eval").readlines()]
    m=min(scores)
    index=[c for c in range(0,len(scores)) if abs(scores[c]-m)<1e-5]
    open(header+".tree",'w').write(string.join([trees[j] for j in index],";\n")+";\n")

    if starttime: print difftime(starttime,timeformat),"Finished extraction of best trees"


def main():
    opts,args=parse_options()  
    Split.infmap={"-":0,"0":0,"1":1} #informative map
    Split.occmap={"-":0,"0":1,"1":1} #map for occuring positions
    mrc(opts)


main()

